package com.mylar.sample.file;

import java.io.File;
import java.util.HashMap;
import java.util.Map;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import org.eclipse.mylar.tasks.core.AbstractAttributeFactory;

public class FileAttributeFactory extends AbstractAttributeFactory
{
	private static final long serialVersionUID = 5333211422546115138L;
	private static final String URL_PREFIX_FILE  = "file:///";

	private Map<String, FileAttribute> attributeByFileRepositoryKey = new HashMap<String, FileAttribute>();
	private Map<String, FileAttribute> attributeByFileRepositoryDataOrder = new HashMap<String, FileAttribute>();
	private Map<String, String> fileRepositoryKeyByTaskKey = new HashMap<String, String>();
	
	public FileAttributeFactory(String repositoryUrl)
	{
		
		if (repositoryUrl.startsWith(URL_PREFIX_FILE) == true)
		{
			String fileName = repositoryUrl.substring(URL_PREFIX_FILE.length());
			if ((fileName.endsWith("/") == false) && (fileName.endsWith("\\") == false))
			{
				fileName = fileName + "/attributes.xml";
			}
			else
			{
				fileName = fileName + "attributes.xml";
			}
			File attrFile = new File(fileName);
			if ((attrFile.isFile() == true) && (attrFile.exists() == true))
			{
				try
				{
					SAXParserFactory parserFactory = SAXParserFactory.newInstance();
					parserFactory.setValidating(true);
					parserFactory.setNamespaceAware(false);
					SAXParser parser = parserFactory.newSAXParser();
		            parser.parse(attrFile, new MyHandler(this));

				}
				catch(Exception ex)
				{
					
				}
			}
		}
	}
	
	public boolean getIsHidden(String key)
	{
		FileAttribute attribute = attributeByFileRepositoryKey.get(key);
		return (attribute != null) ? attribute.isHidden() : false;
	}

	public String getName(String key)
	{
		FileAttribute attribute = attributeByFileRepositoryKey.get(key);
		return (attribute != null) ? attribute.toString() : key;
	}

	public boolean isReadOnly(String key)
	{
		FileAttribute attribute = attributeByFileRepositoryKey.get(key);
		return (attribute != null) ? attribute.isReadOnly() : false;
	}

	public String mapCommonAttributeKey(String key)
	{
		String tracKey = fileRepositoryKeyByTaskKey.get(key);
		return (tracKey != null) ? tracKey : key;
	}
	
	public FileAttribute getArributeByDataOrder(String dataOrder)
	{
		return attributeByFileRepositoryDataOrder.get(dataOrder);
	}
	
	public FileAttribute getArributeByAttributeName(String attributeName)
	{
		return attributeByFileRepositoryKey.get(mapCommonAttributeKey(attributeName));
	}
	
	public class MyHandler extends DefaultHandler
	{
		public MyHandler(FileAttributeFactory attribuetFactory)
		{
		}
		
		public void startElement(String uri, String localName, String qName, org.xml.sax.Attributes attributes)
		throws SAXException
		{
			System.out.println("URI:" + uri);
			System.out.println("LOCALNAME:" + localName);
			System.out.println("qNAME:" + qName);
			
			if (qName.compareToIgnoreCase("Attribute") == 0)
			{
				String name = attributes.getValue("name");
				String description = attributes.getValue("description");
				String mylarname = attributes.getValue("mylarname");
				
				String hidden = attributes.getValue("hidden");
				boolean ishidden = false;
				if ((hidden != null) && (hidden.compareToIgnoreCase("1") == 0))
				{
					ishidden = true;
				}
				String readonly = attributes.getValue("readonly");
				boolean isreadonly = false;
				if ((readonly != null) && (readonly.compareToIgnoreCase("1") == 0))
				{
					isreadonly = true;
				}
				
				String dataorder = attributes.getValue("dataorder");
				
				System.out.println(name + " " + description + " " + hidden + " " + readonly + " " + mylarname);
				
				FileAttribute fileAttribute = new FileAttribute(name, description, mylarname, ishidden, isreadonly);
				
				attributeByFileRepositoryKey.put(name, fileAttribute);
				if (mylarname != null)
				{
					fileRepositoryKeyByTaskKey.put(mylarname, name);
				}
				
				if (dataorder != null)
				{
					attributeByFileRepositoryDataOrder.put(dataorder, fileAttribute);
				}
			}
				
		}
		
		
	}
}
