package com.mylar.sample.file;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.IProgressMonitor;
import java.util.List;


import org.eclipse.mylar.tasks.core.AbstractRepositoryConnector;
import org.eclipse.mylar.tasks.core.TaskRepository;
import org.eclipse.mylar.tasks.core.AbstractRepositoryTask;
import org.eclipse.mylar.tasks.core.IAttachmentHandler;
import org.eclipse.mylar.tasks.core.AbstractRepositoryQuery;
import org.eclipse.mylar.tasks.core.QueryHitCollector;
import org.eclipse.mylar.tasks.core.ITaskDataHandler;
import org.eclipse.mylar.tasks.core.TaskList;
import org.eclipse.mylar.tasks.core.ITask;
import org.eclipse.mylar.tasks.core.RepositoryTaskData;
import org.eclipse.mylar.tasks.ui.TasksUiPlugin;

import com.mylar.sample.Activator;

public class FileRepositoryConnector extends AbstractRepositoryConnector
{
	private final static String CLIENT_LABEL = "Sample File Repository Connector";
	
	
	private FileTaskDataHandler taskDataHandler;

	public void init(TaskList taskList)
	{
		super.init(taskList);
		this.taskDataHandler = new FileTaskDataHandler(this, taskList);
//		this.attachmentHandler = new BugzillaAttachmentHandler(this);
	}

	
/*** START implemented methods ***/

	/**
	 * Returns the repository kind
	 */
	public String getRepositoryType()
	{
		return Activator.FILE_REPOSITORY_KIND;
	}

	/**
	 * Indicates if a new repository task can be created.
	 */
	public boolean canCreateNewTask(TaskRepository repository) {
		return true;
	}

	/**
	 * Returns the instance of 'ITaskDataHandler' used for accessing
	 * data in the associated repository.
	 */
	public ITaskDataHandler getTaskDataHandler()
	{
		return taskDataHandler;
	}

	/**
	 * Returns the label to be displayed in the list of available connectors that can be
	 * created.
	 */
	public String getLabel()
	{
		return CLIENT_LABEL;
	}

	/**
	 * Execute the specified query against the specified repository.
	 * Implementors must execute query synchronously.
	 *
	 * @param query				The query to be executed.
	 * @param repository		The repository to be queried
	 * @param monitor		  	Progress monitor
	 * @param resultCollector	An instance IQueryHitCollector that collects the hits found.
	 */

	public IStatus performQuery(AbstractRepositoryQuery query, TaskRepository repository,
			IProgressMonitor monitor, QueryHitCollector resultCollector)
	{
		if (taskDataHandler != null)
		{
			taskDataHandler.executeQuery(query, resultCollector, taskList);
		}
		return Status.OK_STATUS;
	}
	

/*** END implemented methods ***/
	
	
/***************** Optional methods to be fill in later ****************/	
	

	public IAttachmentHandler getAttachmentHandler()
	{
		return null;
	}

	
	public String getRepositoryUrlFromTaskUrl(String taskFullUrl)
	{
		return null;
	}

	public String getTaskIdFromTaskUrl(String taskFullUrl)
	{
		return null;
	}

	public String getTaskWebUrl(String repositoryUrl, String taskId)
	{
		return null;
	}
	
	public boolean canCreateTaskFromKey(TaskRepository repository)
	{
		return true;
	}


	/**
	 * Updates the properties of <code>repositoryTask</code>. Invoked when on
	 * task synchronization if {@link #getTaskDataHandler()} returns
	 * <code>null</code> or
	 * {@link ITaskDataHandler#getTaskData(TaskRepository, String)} returns
	 * <code>null</code>.
	 * 
	 * <p>
	 * Connectors that provide {@link RepositoryTaskData} objects for all tasks
	 * do not need to implement this method.
	 * 
	 * @param repository
	 *            the repository
	 * @param repositoryTask
	 *            the task that is synchronized
	 * @throws CoreException
	 *             thrown in case of error while synchronizing
	 * @see {@link #getTaskDataHandler()}
	 */
	public void updateTask(TaskRepository repository, AbstractRepositoryTask repositoryTask)
			throws CoreException
	{
		return;
	}
	
	/**
	 * @param id
	 *            identifier, e.g. "123" bug Bugzilla bug 123
	 * @return null if task could not be created
	 * @throws CoreException
	 *             TODO
	 */
	public AbstractRepositoryTask createTaskFromExistingKey(TaskRepository repository, String id)
			throws CoreException
	{
		int taskId = -1;
		try
		{
			if (id != null)
			{
				taskId = Integer.parseInt(id);
			}
			else
			{
				throw new CoreException(new Status(IStatus.ERROR, Activator.PLUGIN_ID, IStatus.OK,
						"invalid task id: null", new Exception("Invalid taskt id: null")));
			}
		}
		catch (NumberFormatException nfe)
		{
			throw new CoreException(new Status(IStatus.ERROR, Activator.PLUGIN_ID, IStatus.OK,
						"invalid task id: " + id, nfe));
		}

		String handle = AbstractRepositoryTask.getHandle(repository.getUrl(), taskId);
		ITask task = taskList.getTask(handle);

		if (task == null)
		{
			RepositoryTaskData taskData = null;
			
			taskData = taskDataHandler.getTaskData(repository, id);
			if (taskData != null)
			{
				task = new FileTask(handle, taskData.getId() + ": " + taskData.getDescription(), true);
				
				((FileTask) task).setTaskData(taskData);
				
				taskList.addTask(task);
			}
		}
		return (AbstractRepositoryTask) task;
	}
	
	/**
	 * Reset and update the repository attributes from the server (e.g.
	 * products, components)
	 *
	 * TODO: remove?
	 */

	public void updateAttributes(TaskRepository repository, IProgressMonitor monitor) throws CoreException
	{
		return;
	}

	public List<String> getSupportedVersions()
	{
		return null;
	}

	/***************** Start repository specific methods ****************/	
	
	public static FileTask createTask(String handleIdentifier, String description)
	{
		FileTask task;
		ITask existingTask = TasksUiPlugin.getTaskListManager().getTaskList().getTask(handleIdentifier);
		if (existingTask instanceof FileTask)
		{
			task = (FileTask) existingTask;
		}
		else
		{
			task = new FileTask(handleIdentifier, description, true);
			TasksUiPlugin.getTaskListManager().getTaskList().addTask(task);
		}
		return task;
	}

}
