package com.mylar.sample.file;

import java.util.Set;
import java.io.File;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.UnsupportedEncodingException;
import java.io.IOException;
import java.util.Date;
import java.util.Calendar;
import java.text.SimpleDateFormat;

import org.eclipse.core.runtime.CoreException;

import org.eclipse.mylar.tasks.core.ITaskDataHandler;
import org.eclipse.mylar.tasks.core.TaskRepository;
import org.eclipse.mylar.tasks.core.RepositoryTaskData;
import org.eclipse.mylar.tasks.core.AbstractRepositoryTask;
import org.eclipse.mylar.tasks.core.AbstractAttributeFactory;
import org.eclipse.mylar.tasks.core.TaskList;
import org.eclipse.mylar.tasks.core.RepositoryTaskAttribute;
import org.eclipse.mylar.tasks.core.AbstractRepositoryQuery;
import org.eclipse.mylar.tasks.core.QueryHitCollector;

import com.mylar.sample.Activator;

/**
 *	This class is used by Mylar to access data the in associated repository.
 */
public class FileTaskDataHandler implements ITaskDataHandler
{
	private static final String URL_PREFIX_FILE  = "file:///";
	private static final String KEY = "key=";
	
	
	private TaskList taskList;

	private FileRepositoryConnector connector;
	
	private static final String DATE_FORMAT_1 = "dd MMM yyyy HH:mm:ss";

	private static SimpleDateFormat modified_ts_format = new SimpleDateFormat(DATE_FORMAT_1);

	/**
	 * Class constructor.  Performs basic initialization for the class.
	 * @param connector
	 * @param taskList
	 */
	public FileTaskDataHandler(FileRepositoryConnector connector, TaskList taskList) {
		this.taskList = taskList;
		this.connector = connector;
	}
	
	
	public RepositoryTaskData getTaskData(TaskRepository repository, String taskId) throws CoreException
	{
		RepositoryTaskData taskData = null;
	
		String[] taskTokens = null;
		
		String fileUrl = repository.getUrl();
		FileAttributeFactory attributeFactory = new FileAttributeFactory(fileUrl);
		if (fileUrl.startsWith(URL_PREFIX_FILE) == true)
		{
			String fileName = fileUrl.substring(URL_PREFIX_FILE.length());
			if ((fileName.endsWith("/") == false) && (fileName.endsWith("\\") == false))
			{
				fileName = fileName + "/tasks.txt";
			}
			else
			{
				fileName = fileName + "tasks.txt";
			}
			File repoFile = new File(fileName);
			if ((repoFile.isFile() == true) && (repoFile.exists() == true))
			{
				try
				{
					FileReader fileReader = new FileReader(repoFile);
					BufferedReader bufferedReader = new BufferedReader(fileReader);
					String taskLine = null;
					String[] tokens = null;
					try
					{
						while (true)
						{
							taskLine = bufferedReader.readLine();
							if (taskLine == null)
							{
								taskTokens = null;
								break;
							}
							if ((taskLine.length() == 0) || (taskLine.startsWith("//") == true))
							{
								taskTokens = null;
								continue;
							}
							taskTokens = taskLine.split(",");
							if (taskTokens[0].trim().compareTo(taskId) == 0)
							{
								break;
							}
							taskTokens = null;
							
						}
						
						bufferedReader.close();
					}
					catch(IOException ex)
					{
						taskLine = null;
						taskTokens = null;
					}
				}
				catch(FileNotFoundException ex)
				{
				}
			}
		}
		
		if (taskTokens != null)
		{
			taskData = new RepositoryTaskData(attributeFactory, Activator.FILE_REPOSITORY_KIND, repository.toString(), "" + taskId);
			addTaskAttributes(attributeFactory, taskData, taskTokens);
		}
		
		return taskData;
	}

	/**
	 * Return a reference to the newly created report in the case of new task submission,
	 * null otherwise
	 */
	public String postTaskData(TaskRepository repository, RepositoryTaskData taskData) throws CoreException
	{
		return null;
	}

	/**
	 * returns all tasks if date is null or an error occurs
	 */
	public Set<AbstractRepositoryTask> getChangedSinceLastSync(TaskRepository repository,
			Set<AbstractRepositoryTask> tasks) throws CoreException, UnsupportedEncodingException
	{
		return null;
	}

	/**
	 * @return null if date cannot be parsed
	 */
	public Date getDateForAttributeType(String attributeKey, String dateString)
	{
		return null;
	}

	public AbstractAttributeFactory getAttributeFactory()
	{
		return null;
//		return attributeFactory;
	}

	/**
	 * Method created to exceute the query entered by the user.  It is called from
	 * the FileRespoitoryConnector.performQuery method.
	 * 
	 * @param query			Query to be executed.
	 * @param collector		Collector to place repository tasks that match the query.
	 * @param taskList
	 */
	public void executeQuery(AbstractRepositoryQuery query, QueryHitCollector collector, TaskList taskList)
	{
		String queryUrl = query.getUrl();
		String repositoryUrl = query.getRepositoryUrl();
		int posn = queryUrl.indexOf("?");
		String queryString = queryUrl.substring(posn +1 );
		String[] queryTokens = queryString.split("&");
		if (queryTokens.length != 0)
		{
			String[] taskTokens = null;
		
			if (repositoryUrl.startsWith(URL_PREFIX_FILE) == true)
			{
				String fileName = repositoryUrl.substring(URL_PREFIX_FILE.length());
				if ((fileName.endsWith("/") == false) && (fileName.endsWith("\\") == false))
				{
					fileName = fileName + "/tasks.txt";
				}
				else
				{
					fileName = fileName + "tasks.txt";
				}
				File repoFile = new File(fileName);
				if ((repoFile.isFile() == true) && (repoFile.exists() == true))
				{
					try
					{
						FileReader fileReader = new FileReader(repoFile);
						BufferedReader bufferedReader = new BufferedReader(fileReader);
						String taskLine = null;
						String[] tokens = null;
						
						try
						{
							String keyValue = queryTokens[0].substring(KEY.length());
							if (keyValue.compareTo("*") == 0)
							{
								while (true)
								{
									taskLine = bufferedReader.readLine();
									if (taskLine == null)
									{
										taskTokens = null;
										break;
									}
									if ((taskLine.length() == 0) || (taskLine.startsWith("//") == true))
									{
										taskTokens = null;
										continue;
									}
									taskTokens = taskLine.split(",");
									FileQueryHit queryHit = new FileQueryHit(taskList,  taskTokens[1].trim(), taskTokens[2].trim(), repositoryUrl, taskTokens[0].trim(), "OPEN");
									collector.accept(queryHit);
									taskTokens = null;
								}
							}
							else
							{
								while (true)
								{
									taskLine = bufferedReader.readLine();
									if (taskLine == null)
									{
										break;
									}
									if ((taskLine.length() == 0) || (taskLine.startsWith("//") == true))
									{
										taskTokens = null;
										continue;
									}
									taskTokens = taskLine.split(",");
									if (taskTokens[0].compareTo(keyValue) == 0)
									{
										FileQueryHit queryHit = new FileQueryHit(taskList,  taskTokens[1].trim(), taskTokens[2].trim(), repositoryUrl, taskTokens[0].trim(), "OPEN");
										collector.accept(queryHit);
										break;
									}
									taskTokens = null;
								}
							}
							
							bufferedReader.close();
						}
						catch(IOException ex)
						{
							taskLine = null;
							taskTokens = null;
						}
						
						bufferedReader.close();
					}
					catch(FileNotFoundException ex)
					{
					}
					catch(IOException ex)
					{
					}
					catch(CoreException ex)
					{
					}
				}
			}
		}
	}
	
	private void addTaskAttributes(FileAttributeFactory attributeFactory, RepositoryTaskData taskData, String[] taskDataValues)
	{
		RepositoryTaskAttribute attribute = null;
		FileAttribute fileAttribute = null;
		
		if (taskDataValues != null)
		{
			for (int idx = 0; idx < taskDataValues.length; idx++)
			{
				fileAttribute = attributeFactory.getArributeByDataOrder(""+idx);
				if (fileAttribute != null)
				{
					attribute = attributeFactory.createAttribute(fileAttribute.getAttributeName());
					attribute.setValue(taskDataValues[idx].trim());
					attribute.setReadOnly(fileAttribute.isReadOnly());
					taskData.addAttribute(fileAttribute.getAttributeName(), attribute);
				}
			}
			
			fileAttribute = attributeFactory.getArributeByAttributeName("file.owner");
			if (fileAttribute != null)
			{
				attribute = attributeFactory.createAttribute(fileAttribute.getAttributeName());
				attribute.setValue(System.getProperty("user.name","<unknown>"));
				attribute.setReadOnly(fileAttribute.isReadOnly());
				taskData.addAttribute(fileAttribute.getAttributeName(), attribute);
			}
			
			fileAttribute = attributeFactory.getArributeByAttributeName("file.version");
			if (fileAttribute != null)
			{
				attribute = attributeFactory.createAttribute(fileAttribute.getAttributeName());
				attribute.setValue("11.0.0.376");
				attribute.setReadOnly(fileAttribute.isReadOnly());
				taskData.addAttribute(fileAttribute.getAttributeName(), attribute);
			}
			
			try
			{
				Date modified = modified_ts_format.parse("19 Dec 2006 03:00:00 EST");
				String xxx = modified.toGMTString();
				
				fileAttribute = attributeFactory.getArributeByAttributeName("file.created");
				if (fileAttribute != null)
				{
					attribute = attributeFactory.createAttribute(fileAttribute.getAttributeName());
					attribute.setValue(modified.toGMTString());
					attribute.setReadOnly(fileAttribute.isReadOnly());
					taskData.addAttribute(fileAttribute.getAttributeName(), attribute);
				}

				fileAttribute = attributeFactory.getArributeByAttributeName("file.modified");
				if (fileAttribute != null)
				{
					attribute = attributeFactory.createAttribute(fileAttribute.getAttributeName());
					attribute.setValue(modified.toGMTString());
					attribute.setReadOnly(fileAttribute.isReadOnly());
					taskData.addAttribute(fileAttribute.getAttributeName(), attribute);
				}
			}
			catch(Exception ex)
			{
				int jnwx = 0;
			}
			
		}
	}
}
