package com.mylar.sample.file.ui;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.core.runtime.IProgressMonitor;

import org.eclipse.mylar.tasks.ui.TasksUiPlugin;
import org.eclipse.mylar.tasks.core.TaskRepository;
import org.eclipse.mylar.tasks.ui.search.AbstractRepositoryQueryPage;
import org.eclipse.mylar.context.core.MylarStatusHandler;
import org.eclipse.mylar.tasks.core.AbstractRepositoryConnector;

import com.mylar.sample.Activator;
import com.mylar.sample.file.FileRepositoryQuery;

/**
 * This class is the query wizard.  It allows for the creation of a
 * multi-page wizard to create queries.  It is created by the
 * FileRepositoryUI.getQueryWizard method when Mylar wants to create
 * a repository query.
 */
public class FileRepositoryQueryWizard extends Wizard
{
	private static final String TITLE = "New File Repository Query";

	private final TaskRepository repository;

	FileRepositorySearchPage page1;

	/**
	 * Class constructor.  Performs basic initialzation for the class instance.
	 * 
	 * @param repository	The repository to be queried.
	 */
	public FileRepositoryQueryWizard(TaskRepository repository)
	{
		this.repository = repository;
		setNeedsProgressMonitor(true);
		setWindowTitle(TITLE);
	}

	/**
	 * Add the required wizard pages.
	 */
	public void addPages()
	{
		page1 = new FileRepositorySearchPage(repository);
		page1.setWizard(this);
		addPage(page1);

	}

	/**
	 * Indicates if the wizard can finish.
	 */
	public boolean canFinish()
{
		return true;
	}
	
	/**
	 * Perform the required processing when the user clicks the finish button.
	 *
	 * For This class that means executing the query that was created.
	 * 
	 */
	public boolean performFinish()
	{
		AbstractRepositoryQueryPage page;


		//
		//	Get the created query
		//
		final FileRepositoryQuery queryCategory = (FileRepositoryQuery) page1.getQuery();

		//
		//	Add the query to Mylar's task list
		//
		TasksUiPlugin.getTaskListManager().getTaskList().addQuery(queryCategory);
		
		//
		//	Create a job and execute the query.
		//
		WorkspaceModifyOperation op = new WorkspaceModifyOperation() {
				protected void execute(IProgressMonitor monitor) throws CoreException {
					monitor.beginTask("Executing query", 50);
					try {
						AbstractRepositoryConnector connector = TasksUiPlugin.getRepositoryManager()
								.getRepositoryConnector(Activator.FILE_REPOSITORY_KIND);
						TasksUiPlugin.getSynchronizationManager().synchronize(connector, queryCategory, null);
					} finally {
						monitor.done();
					}
				}
			};

		try
		{
			getContainer().run(true, false, op);
		} catch (Exception e) {
			MylarStatusHandler.log(e, "There was a problem executing the query refresh.");
		}
	
		return true;
		
	}
}
