package com.mylar.sample.file.ui;

import java.io.File;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.jface.dialogs.MessageDialog;

import org.eclipse.mylar.tasks.ui.wizards.AbstractRepositorySettingsPage;
import org.eclipse.mylar.tasks.ui.AbstractRepositoryConnectorUi;
import org.eclipse.mylar.tasks.core.RepositoryTemplate;

/**
 *	This class is used by Mylar to display and process the settings required
 *	to create a repository
 */
public class FileRepositorySettingsPage extends AbstractRepositorySettingsPage {

	private static final String URL_PREFIX_FILE  = "file:///";
	
	private static final String TITLE = "Sample File Repository Settings";

	private static final String DESCRIPTION = "Example: file:///c:/TasksDirectory/MyTasks)";
	
	
	/**
	 * Class constructor.  Used to set the title and descrition of the
	 * respository.
	 * 
	 * @param repositoryUi	The repository connector UI
	 */
	public FileRepositorySettingsPage(AbstractRepositoryConnectorUi repositoryUi)
	{
		super(TITLE, DESCRIPTION, repositoryUi);
		this.
		setNeedsAnonymousLogin(false);
		setNeedsEncoding(true);
		setNeedsTimeZone(false);
		setNeedsHttpAuth(false);
	}

	/**
	 * This method is used to add additional fields to the connector settings
	 */
	protected void createAdditionalControls(Composite parent) {

		/**
		 * Add the name of any connector templates defined in the
		 * plugin.xml file.
		 */
//		for (RepositoryTemplate template : connector.getTemplates()) {
//			serverUrlCombo.add(template.label);
//		}
	}

	/**
	 * This method is called then the user clicks the "validate" button on the
	 * connector settings
	 */
	protected void validateSettings()
	{
		final String serverUrl = getServerUrl();
		final String label = getName();
		final String newUserId = getUserName();
		final String newPassword = getPassword();
		
		if ((label == null) || (label.length() == 0))
		{
			MessageDialog.openError(null, "Sample File Connector Error", "The File repository requires a name.");
		}
		
		if ((serverUrl == null) || (serverUrl.length() == 0) || (isValidUrl(serverUrl) == false))
		{
			MessageDialog.openError(null, "Sample File Connector Error", "Invalid file repository url: " + serverUrl);
		}
		
		//
		//	Validate required files are present
		//
		boolean bValid = validateRequiredFiles(serverUrl);
		if (bValid == true)
		{
			MessageDialog.openInformation(null, "Sample File Connector", "Repository '" + serverUrl + "' is a valid file repository.");
		}
		
		return;
	}

	/**
	 * This method is called when the user enters information in the "server" field
	 * of the connection settings.  It is called after every key stroke.
	 */
	protected boolean isValidUrl(String name)
	{
		boolean bIsValid = false;
		
		if (name.startsWith(URL_PREFIX_FILE) == true)
		{
			String fileName = name.substring(URL_PREFIX_FILE.length());
			File repoFile = new File(fileName);
			if ((repoFile.exists() == true) && (repoFile.isDirectory() == true) )
			{
				bIsValid = true;
			}
		}
		
		return bIsValid;
	}
	
	/**
	 * Method used to validate the files required to this respository.
	 * 
	 * @param repositoryUrl	Url of the repository.
	 * @return
	 */
	private boolean validateRequiredFiles(String repositoryUrl)
	{
		boolean bFileExists = false;

		if (repositoryUrl.startsWith(URL_PREFIX_FILE) == true)
		{
			String fileName = repositoryUrl.substring(URL_PREFIX_FILE.length());
			File repoFile = new File(fileName);
			if ((repoFile.exists() == true) && (repoFile.isDirectory() == true) )
			{
				//
				//	Check if 'tasks.txt' exists
				//
				String tasksFileName = null;
				if ((fileName.endsWith("/") == false) && (fileName.endsWith("\\") == false))
				{
					tasksFileName = fileName + "/tasks.txt";
				}
				else
				{
					tasksFileName = fileName + "tasks.txt";
				}
				
				File tasksFile = new File(tasksFileName);
				if ((tasksFile.exists() == false) || (tasksFile.isFile() == false))
				{
					MessageDialog.openError(null, "Sample File Connector Error", "Required file 'tasks.txt' does not exists in repository: " + repositoryUrl);
					return false;
				}
				
				//
				//	Check if 'attributes.txt' exists
				//
				String attributesFileName = null;
				if ((fileName.endsWith("/") == false) && (fileName.endsWith("\\") == false))
				{
					attributesFileName = fileName + "/attributes.xml";
				}
				else
				{
					attributesFileName = fileName + "attributes.xml";
				}
				
				File attributesFile = new File(attributesFileName);
				if ((attributesFile.exists() == false) || (attributesFile.isFile() == false))
				{
					MessageDialog.openError(null, "Sample File Connector Error", "Required file 'attributes.txt' does not exists in repository: " + repositoryUrl);
					return false;
				}
				
				//
				//	Indicate repository contains required files
				//
				bFileExists = true;
			}
			else
			{
				MessageDialog.openError(null, "Sample File Connector Error", "Repository '" + repositoryUrl + "' does not exist.");
			}
		}
		
		return bFileExists;
	}
}
