package com.mylar.sample.file.ui;


import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.mylar.tasks.core.AbstractRepositoryQuery;
import org.eclipse.mylar.tasks.core.TaskRepository;
import org.eclipse.mylar.tasks.ui.AbstractRepositoryConnectorUi;
import org.eclipse.mylar.tasks.ui.search.AbstractRepositoryQueryPage;
import org.eclipse.mylar.tasks.ui.wizards.AbstractRepositorySettingsPage;

import com.mylar.sample.Activator;
import com.mylar.sample.file.ui.FileRepositoryNewTaskWizard;
import com.mylar.sample.file.ui.FileRepositorySearchPage;

/**
 * This class is used by Mylar to determine what GUI interfaces the user
 * repository support.s
*/
public class FileRepositoryUI extends AbstractRepositoryConnectorUi
{

	/**
	 * Get an instance of the page that allows for the creation and seeting of
	 * repository properties
	 */
	public AbstractRepositorySettingsPage getSettingsPage()
	{
		return new FileRepositorySettingsPage(this);
	}

	/**
	 * Get an instance of tge page that allows for the user to perform searches
	 */
	public AbstractRepositoryQueryPage getSearchPage(TaskRepository repository, IStructuredSelection selection)
	{
		return new FileRepositorySearchPage(repository);
	}

	public IWizard getNewTaskWizard(TaskRepository taskRepository) {
		return new FileRepositoryNewTaskWizard(taskRepository);
	}

	/**
	 * Get an instance of the wizard used to create respository queries.
	 */
	public IWizard getQueryWizard(TaskRepository repository, AbstractRepositoryQuery query){
		return new FileRepositoryQueryWizard(repository);
		}

	/**
	 * Returns an indication if the repository supports searchs
	 */
	public boolean hasSearchPage()
	{
		return true;
	}

	/**
	 * Returns an indication of the repository as a RichEditor
	 */
	public boolean hasRichEditor()
	{
		return true;
	}

	/**
	 * Returns the repository kind
	 */
	public String getRepositoryType()
	{
		return Activator.FILE_REPOSITORY_KIND;
	}

	/**
	 * Returns an indication if a remote task was opened.
	 */
	public boolean openRemoteTask(String repositoryUrl, String idString) {
		return false;
	}

}
