package com.mylar.sample.file.ui.editor;

import org.eclipse.mylar.context.core.MylarStatusHandler;
import org.eclipse.mylar.tasks.core.ITask;
import org.eclipse.mylar.tasks.core.TaskRepository;
import org.eclipse.mylar.tasks.ui.TasksUiPlugin;
import org.eclipse.mylar.tasks.ui.editors.ITaskEditorFactory;
import org.eclipse.mylar.tasks.ui.editors.RepositoryTaskEditorInput;
import org.eclipse.mylar.tasks.ui.editors.TaskEditor;
import org.eclipse.mylar.tasks.ui.editors.TaskEditorInput;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;

import com.mylar.sample.Activator;
import com.mylar.sample.file.FileTask;
import com.mylar.sample.file.ui.editor.FileTaskEditor;

public class FileTaskEditorFactory implements ITaskEditorFactory
{

	public boolean canCreateEditorFor(ITask task)
	{
		return task instanceof FileTask;
	}

	public boolean canCreateEditorFor(IEditorInput input)
	{
		if (input instanceof RepositoryTaskEditorInput)
		{
			RepositoryTaskEditorInput existingInput = (RepositoryTaskEditorInput) input;
			return existingInput.getTaskData() != null
					&& Activator.FILE_REPOSITORY_KIND.equals(existingInput.getRepository().getKind());
		} 
		return false;
	}

	public IEditorPart createEditor(TaskEditor parentEditor, IEditorInput editorInput) {
		if (editorInput instanceof RepositoryTaskEditorInput  || editorInput instanceof TaskEditorInput) {
			return new FileTaskEditor(parentEditor);
		} 
		return null;
	}

	public IEditorInput createEditorInput(ITask task)
	{
		FileTask jiraTask = (FileTask) task;
		TaskRepository repository = TasksUiPlugin.getRepositoryManager().getRepository(Activator.FILE_REPOSITORY_KIND,
				jiraTask.getRepositoryUrl());
		try
		{
			return new RepositoryTaskEditorInput(repository, jiraTask.getHandleIdentifier(), jiraTask.getUrl());
		}
		catch (Exception e)
		{
			MylarStatusHandler.fail(e, "Could not create Sample File Repositor editor input", true);
		}
		return null;
	}

	public String getTitle()
	{
		return "Sample File Repository";
	}

	public boolean providesOutline() 
	{
		return true;
	}
}
